﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OZX
{
    public class OzPDU_OZX
    {
        //**********************************
        // Login
        //**********************************

        public const byte Login = 1;
        public const byte Login_Resp = 2;

        //**********************************
        // Keep link alive
        //**********************************

        public const byte Keepalive = 3;
        public const byte Keepalive_Resp = 4;

        //**********************************
        // Send messages
        //**********************************

        public const byte Send = 5;
        public const byte Send_Resp = 6;

        public const byte Send_Success = 7;
        public const byte Send_Success_Resp = 8;

        public const byte Send_Failed = 9;
        public const byte Send_Failed_Resp = 10;

        public const byte Send_DeliverySuccess = 11;
        public const byte Send_DeliverySuccess_Resp = 12;

        public const byte Send_DeliveryFailed = 13;
        public const byte Send_DeliveryFailed_Resp = 14;

        public const byte Send_Viewed = 15;
        public const byte Send_Viewed_Resp = 16;


        //**********************************
        // Command
        //**********************************

        public virtual byte GetCommandID()
        {
            return 0; 
        }

        //**********************************
        // Encode
        //**********************************

        public byte[] Encode()
        {
            var data = EncodeData();
            var dataLen = OzCommonBytes.IntToBytes(data.Length);

            var ret = new byte[data.Length + 5];
            //1 byte: command
            ret[0] = GetCommandID();
            //4 bytes: length
            Buffer.BlockCopy(dataLen, 0, ret, 1, 4);
            //N bytes: data
            Buffer.BlockCopy(data, 0, ret, 5, data.Length);

            return ret;
        }

        protected virtual byte[] EncodeData()
        {
            return new byte[0];
        }

        //**********************************
        // Decode
        //**********************************

        public static OzPDU_OZX Decode(byte[] src, ref int offset)
        {
            if (src.Length < 1)
                return null;

            //Identify command
            byte command = src[offset++];
            var commandInstance = GetCommandInstance(command);

            //decode data
            int dataLen = OzCommonBytes.BytesToInt(src, ref offset);
            if (src.Length < offset + dataLen)
            {
                return null;
            }

            DecodeData(src, ref offset, commandInstance);
            return commandInstance;
        }

        public static void DecodeData(byte[] src, ref int offset, OzPDU_OZX commandInstance)
        {
            commandInstance.DecodeData(src, ref offset);
        }

        public static OzPDU_OZX GetCommandInstance(byte command)
        {
            switch (command)
            {
                case Login:
                    return new OzPDU_OZX_Login();
                case Login_Resp:
                    return new OzPDU_OZX_Login_Resp();
        
                case Keepalive:
                    return new OzPDU_OZX_Keepalive();
                case Keepalive_Resp:
                    return new OzPDU_OZX_Keepalive_Resp();

                case Send:
                    return new OzPDU_OZX_Send();
                case Send_Resp:
                    return new OzPDU_OZX_Send_Resp();

                case Send_Success:
                    return new OzPDU_OZX_Send_SubmitSuccess();
                case Send_Success_Resp:
                    return new OzPDU_OZX_Send_SubmitSuccess_Resp();

                case Send_Failed:
                    return new OzPDU_OZX_Send_SubmitFailed();
                case Send_Failed_Resp:
                    return new OzPDU_OZX_Send_SubmitFailed_Resp();

                case Send_DeliverySuccess:
                    return new OzPDU_OZX_Send_DeliverySuccess();
                case Send_DeliverySuccess_Resp:
                    return new OzPDU_OZX_Send_DeliverySuccess_Resp();

                case Send_DeliveryFailed:
                    return new OzPDU_OZX_Send_DeliveryFailed();
                case Send_DeliveryFailed_Resp:
                    return new OzPDU_OZX_Send_DeliveryFailed_Resp();

                case Send_Viewed:
                    return new OzPDU_OZX_Send_Viewed();
                case Send_Viewed_Resp:
                    return new OzPDU_OZX_Send_Viewed_Resp();
            }
            return null;
        }

        protected virtual void DecodeData(byte[] src, ref int offset)
        {
            return;
        }

    }
}
